@@ -3,13 +3,8 @@ from __future__ import annotations
 from typing import TYPE_CHECKING, Any
 
 from narwhals._pandas_like.utils import select_columns_by_name
-from narwhals._utils import (
-    Implementation,
-    Version,
-    isinstance_or_issubclass,
-    parse_version,
-)
-from narwhals.dependencies import get_pandas, get_pyarrow
+from narwhals._utils import Implementation, Version, isinstance_or_issubclass
+from narwhals.dependencies import get_pyarrow
 
 if TYPE_CHECKING:
     from collections.abc import Sequence
@@ -17,7 +12,7 @@ if TYPE_CHECKING:
     import dask.dataframe as dd
     import dask.dataframe.dask_expr as dx
 
-    from narwhals._dask.dataframe import DaskLazyFrame
+    from narwhals._dask.dataframe import DaskLazyFrame, Incomplete
     from narwhals._dask.expr import DaskExpr
     from narwhals.typing import IntoDType
 else:
@@ -58,19 +53,13 @@ def align_series_full_broadcast(
     ]  # pyright: ignore[reportReturnType]
 
 
-def add_row_index(
-    frame: dd.DataFrame,
-    name: str,
-    backend_version: tuple[int, ...],
-    implementation: Implementation,
-) -> dd.DataFrame:
+def add_row_index(frame: dd.DataFrame, name: str) -> dd.DataFrame:
     original_cols = frame.columns
-    frame = frame.assign(**{name: 1})
+    df: Incomplete = frame.assign(**{name: 1})
     return select_columns_by_name(
-        frame.assign(**{name: frame[name].cumsum(method="blelloch") - 1}),
+        df.assign(**{name: df[name].cumsum(method="blelloch") - 1}),
         [name, *original_cols],
-        backend_version,
-        implementation,
+        Implementation.DASK,
     )
 
 
@@ -113,7 +102,7 @@ def narwhals_to_native_dtype(dtype: IntoDType, version: Version) -> Any:  # noqa
     if isinstance_or_issubclass(dtype, dtypes.UInt8):
         return "uint8"
     if isinstance_or_issubclass(dtype, dtypes.String):
-        if (pd := get_pandas()) is not None and parse_version(pd) >= (2, 0, 0):
+        if Implementation.PANDAS._backend_version() >= (2, 0, 0):
             if get_pyarrow() is not None:
                 return "string[pyarrow]"
             return "string[python]"  # pragma: no cover
@@ -130,7 +119,7 @@ def narwhals_to_native_dtype(dtype: IntoDType, version: Version) -> Any:  # noqa
             # NOTE: `pandas-stubs.core.dtypes.dtypes.CategoricalDtype.categories` is too narrow
             # Should be one of the `ListLike*` types
             # https://github.com/pandas-dev/pandas-stubs/blob/8434bde95460b996323cc8c0fea7b0a8bb00ea26/pandas-stubs/_typing.pyi#L497-L505
-            return pd.CategoricalDtype(dtype.categories, ordered=True)  # pyright: ignore[reportArgumentType]
+            return pd.CategoricalDtype(dtype.categories, ordered=True)  # type: ignore[arg-type]
         msg = "Can not cast / initialize Enum without categories present"
         raise ValueError(msg)
 
@@ -144,19 +133,19 @@ def narwhals_to_native_dtype(dtype: IntoDType, version: Version) -> Any:  # noqa
         return "timedelta64[ns]"
     if isinstance_or_issubclass(dtype, dtypes.List):  # pragma: no cover
         msg = "Converting to List dtype is not supported yet"
-        return NotImplementedError(msg)
+        raise NotImplementedError(msg)
     if isinstance_or_issubclass(dtype, dtypes.Struct):  # pragma: no cover
         msg = "Converting to Struct dtype is not supported yet"
-        return NotImplementedError(msg)
+        raise NotImplementedError(msg)
     if isinstance_or_issubclass(dtype, dtypes.Array):  # pragma: no cover
         msg = "Converting to Array dtype is not supported yet"
-        return NotImplementedError(msg)
+        raise NotImplementedError(msg)
     if isinstance_or_issubclass(dtype, dtypes.Time):  # pragma: no cover
         msg = "Converting to Time dtype is not supported yet"
-        return NotImplementedError(msg)
+        raise NotImplementedError(msg)
     if isinstance_or_issubclass(dtype, dtypes.Binary):  # pragma: no cover
         msg = "Converting to Binary dtype is not supported yet"
-        return NotImplementedError(msg)
+        raise NotImplementedError(msg)
 
     msg = f"Unknown dtype: {dtype}"  # pragma: no cover
     raise AssertionError(msg)
